# Programmer used for In System Programming
ISP_PROG = dapa
# device the ISP programmer is connected to
ISP_DEV = /dev/parport0
# Programmer used for serial programming (using the bootloader)
SERIAL_PROG = butterfly
# device the serial programmer is connected to
SERIAL_DEV = /dev/ttyS0
# device name (for avr-gcc)
MCU = atmega8

# programs
CC = avr-gcc
OBJCOPY = avr-objcopy
OBJDUMP = avr-objdump
AS = avr-as
CP = cp
RM = rm -f
AVRDUDE = avrdude

-include config.mk

# flags for the compiler
CFLAGS += -g -Os -finline-limit=800 -mmcu=$(MCU)

ifneq ($(DEBUG),)
	CFLAGS += -Wall -W -Wchar-subscripts -Wmissing-prototypes
	CFLAGS += -Wmissing-declarations -Wredundant-decls
	CFLAGS += -Wstrict-prototypes -Wshadow -Wbad-function-cast
	CFLAGS += -Winline -Wpointer-arith -Wsign-compare
	CFLAGS += -Wunreachable-code -Wdisabled-optimization -Werror
	CFLAGS += -Wcast-align -Wwrite-strings -Wnested-externs -Wundef
	CFLAGS += -Wa,-adhlns=$(basename $@).lst
	CFLAGS += -DDEBUG
endif

all:

clean:
	$(RM) *.hex *.eep.hex *.o *.lst *.lss

interactive-isp:
	$(AVRDUDE) -p m8 -c $(ISP_PROG) -P $(ISP_DEV) -t

interactive-serial:
	$(AVRDUDE) -p m8 -c $(SERIAL_PROG) -P $(SERIAL_DEV) -t

fuse:
	$(AVRDUDE) -p m8 -c $(ISP_PROG) -P $(ISP_DEV) -U hfuse:w:0xD0:m
	$(AVRDUDE) -p m8 -c $(ISP_PROG) -P $(ISP_DEV) -U lfuse:w:0xE0:m

install-bootloader: bootloader.hex program-isp-bootloader

bootloader.hex:
	cd boot/v0_7 && make
	cp boot/v0_7/main.hex bootloader.hex


.PHONY: all clean interactive-isp interactive-serial

program-isp-%: %.hex
	$(AVRDUDE) -p m8 -c $(ISP_PROG) -P $(ISP_DEV) -U f:w:$<

program-isp-eeprom-%: %.eep.hex
	$(AVRDUDE) -p m8 -c $(ISP_PROG) -P $(ISP_DEV) -U e:w:$<

program-serial-%: %.hex
	$(AVRDUDE) -p m8 -c $(SERIAL_PROG) -P $(SERIAL_DEV) -U f:w:$<

program-serial-eeprom-%: %.eep.hex
	$(AVRDUDE) -p m8 -c $(SERIAL_PROG) -P $(SERIAL_DEV) -U e:w:$<

%.hex: %
	$(OBJCOPY) -O ihex -R .eeprom $< $@

%.eep.hex: %
	$(OBJCOPY) --set-section-flags=.eeprom="alloc,load" --change-section-lma .eeprom=0 -O ihex -j .eeprom $< $@

%.lss: %
	$(OBJDUMP) -h -S $< > $@
